      SUBROUTINE PROP(P,T,ISAT,ITYPE,IFLAG,NMAX,ERRTOL,ERRMSG,ADJ)

C      VERSION 1.03
C      Modifications made to the CLIPP and CLIPT routines only. 
C      EACS/RJW                                     APRIL 1991
*************************************************************************     
VERSION 0.0
*     THIS PROGRAM GENERATES H2O PROPERTIES 
************************************************************************
*     INPUT FLAGS:
*
*                              ITYPE
*                 0    1     2     3     4
*       ISAT   
*       ----------------------------------------------------------------
*         0           based on (P,T) independent of ITYPE
*
*       1           (P,??)  (P,RHO)  (P,h)   (P,x)   (P,void)
*       2         (??,T)  (T,RHO)  (T,h)   (T,x)   (T,void)
*
*         3       based on (RHO,h) independent of ITYPE  
*         4       based on (RHO,P) independent of ITYPE  
*         5           based on (RHO,T) independent of ITYPE  
*         6       based on (h,P) independent of ITYPE  
*         7           based on (h,T) independent of ITYPE  
*    ?? MEANS DON'T CARE
*
*     OUTPUT FLAG:
*           IFLAG - 0=SUBCOOLED  
*                 1=SATURATED
*                 2=SUPERHEATED
************************************************************************
*     INITIALIZATION
************************************************************************
        COMMON/PROPERTY/VFP,VGP,RHOF,RHOG,HFP,HGP,SFP,SGP,EFP,EGP,
     *                  VOL,TOTM,TOTH,TOTS,TOTE,X,VOID,RHO,SPECVOL,
     *                  SPECH,SPECS,SPECE,PS,TS
      COMMON/SLOPES/SRFDP,SRFDT,SRGDP,SRGDT,SHFDP,SHFDT,
     *                  SHGDP,SHGDT,G1P,G2P,G1T,G2T,F1,F2,F3,F4,F5
     *                  ,STDP,SPDT,SRDP,SRDT,SHDP,SHDT
     *            ,SSFDP,SSFDT,SSGDP,SSGDT,SSDP,SSDT      
      COMMON/ERRORS/ERRH2O
      CHARACTER ERRMSG*70, ERRH2O*70
      ERRMSG=' '
      ERRH2O=' '
************************************************************************
*     THE SIMPLEST CASE IS WHEN ISAT=0: P & T ARE GIVEN, THE FLUID IS
*       ASSUMED TO NOT BE SATURATED AND THE PROPERTIES ARE CALCULATED
*       DIRECTLY. THERE IS A CHANCE THAT THE FLUID WILL TURN OUT TO BE
*       SATURATED. IF THIS IS SO, THEN ISAT IS SET TO 1 AND PROPERTY
*       EVALUATION IS DEFERRED TILL LATER (THE CASE OF ISAT=1, WITH ITYPE
*       AND VAR USED AS PREVIOUSLY DEFINED (DEFAULT=0).
************************************************************************
*       FIND THE SATURATION POINTS CORRESPONDING TO THE INPUT T & P
************************************************************************
      PS=PSAT(T)
      TS=TSAT(P)
      CALL CLIPP(P)
      CALL CLIPT(T)
      CALL CLIPH(SPECH)
      CALL CLIPR(RHO)
************************************************************************
*     P & T GIVEN
************************************************************************
      IF (ISAT.EQ.0) THEN
*     CALL SUBCOOLED ROUTINE
            IF (P.GT.PS) THEN
                  IFLAG=0    
                  CALL SUB(P,T)
*     CALL SUPERHEATED ROUTINE
            ELSEIF (P.LT.PS) THEN
                  IFLAG=2    
                  CALL SUP(P,T)
            ELSE
*     RARE CASE OF SATURATED FLUID FOR GIVEN P &T
            ISAT=1
            ENDIF
************************************************************************
*     SATURATED BASED ON P
************************************************************************
      ELSEIF (ISAT.EQ.1) THEN
            IFLAG=1
            T=TSAT(P)
            CALL SAT(P,ITYPE)
************************************************************************
*     SATURATED BASED ON T
************************************************************************
      ELSEIF (ISAT.EQ.2) THEN
            IFLAG=1
            P=PSAT(T)
            CALL SAT(P,ITYPE)
************************************************************************
*     RHO & h GIVEN
************************************************************************
      ELSEIF (ISAT.EQ.3) THEN
            N=0
            NOGO=0
  20        N=N+1
            HGP=HG(P)
            HFP=HF(P)
            IF(SPECH.LT.HFP) THEN
                  IFLAG=0
                  RHO1=DFSUB(P,T)
                  SPECH1=HFSUB(P,T)
                  DR=RHO-RHO1
                  DH=SPECH-SPECH1
                  CALL GSUB(P,T,GG1P,GG2P,GG1T,GG2T)
                  DP=GG1P*DR+GG2P*DH
                  DT=GG1T*DR+GG2T*DH
                  P=P+DP*ADJ
                  T=T+DT*ADJ
            ELSEIF(SPECH.GT.HGP) THEN
                  IFLAG=2
                  RHO1=DGSUP(P,T)
                  SPECH1=HGSUP(P,T)
                  DR=RHO-RHO1
                  DH=SPECH-SPECH1
                  CALL GSUP(P,T,GG1P,GG2P,GG1T,GG2T)
                  DP=GG1P*DR+GG2P*DH
                  DT=GG1T*DR+GG2T*DH
                  P=P+DP*ADJ
                  T=T+DT*ADJ
            ELSE             
                  IFLAG=1
                  SPECH0=SPECH
                  CALL SAT(P,1)
                  SPECH1=SPECH
                  SPECH=SPECH0
                  DH=SPECH-SPECH1
                  DP=DH/SHDP
                  DT=0.
                  P=P+DP*ADJ
                  T=TSAT(P)
            ENDIF
      CALL CLIPP(P)
      CALL CLIPT(T)
      ERROR=AMAX1(ABS(DR/RHO),ABS(DH/SPECH),ABS(DP/P),ABS(DT/T))
            IF(ERROR.LE.ERRTOL) NOGO=1
            IF(N.GE.NMAX) THEN
                  NOGO=1
                  ERRMSG=' MAXIMUM NUMBER OF ITERATIONS USED'
            ENDIF
            IF (NOGO.EQ.0) GOTO 20
            IF (IFLAG.EQ.0) CALL SUB(P,T)
            IF (IFLAG.EQ.1) CALL SAT(P,1)
            IF (IFLAG.EQ.2) CALL SUP(P,T)
************************************************************************
*     RHO & P GIVEN
************************************************************************
      ELSEIF (ISAT.EQ.4) THEN
            N=0
            NOGO=0
            IF(RHO.GT.DF(P)) THEN
  30              N=N+1
                  IFLAG=0
                  RHO1=DFSUB(P,T)
                  DR=RHO-RHO1
                  DT=DR/DRFDTP(P,T)
                  ERROR=AMAX1(ABS(DR/RHO),ABS(DT/T))
                  IF(ERROR.LE.ERRTOL) NOGO=1
                  IF(N.GE.NMAX) THEN
                       NOGO=1
                  ERRMSG=' MAXIMUM NUMBER OF ITERATIONS USED'
                  ENDIF
                  T=T+DT*ADJ
                  CALL CLIPT(T)
                  IF (NOGO.EQ.0) GOTO 30
                  CALL SUB(P,T)
            ELSEIF(RHO.LT.DG(P)) THEN
  40              N=N+1
                  IFLAG=2
                  RHO1=DGSUP(P,T)
                  DR=RHO-RHO1
                  DT=DR/DRGDTP(P,T)
                  ERROR=AMAX1(ABS(DR/RHO),ABS(DT/T))
                  IF(ERROR.LE.ERRTOL) NOGO=1
                  IF(N.GE.NMAX) THEN
                       NOGO=1
                  ERRMSG=' MAXIMUM NUMBER OF ITERATIONS USED'
                  ENDIF
                  T=T+DT*ADJ
                  CALL CLIPT(T)
                  IF (NOGO.EQ.0) GOTO 40
                  CALL SUP(P,T)
            ELSE
                  T=TSAT(P)
                  IFLAG=1
                  CALL SAT(P,1)
            ENDIF
************************************************************************
*     RHO & T GIVEN
************************************************************************
      ELSEIF (ISAT.EQ.5) THEN
            PS=PSAT(T)
            N=0
            NOGO=0
            IF(RHO.GT.DF(PS)) THEN
  50              N=N+1
                  IFLAG=0
                  RHO1=DFSUB(P,T)
                  DR=RHO-RHO1
                  DP=DR/DRFDPT(P,T)
                  ERROR=AMAX1(ABS(DR/RHO),ABS(DP/P))
                  IF(ERROR.LE.ERRTOL) NOGO=1
                  IF(N.GE.NMAX) THEN
                       NOGO=1
                  ERRMSG=' MAXIMUM NUMBER OF ITERATIONS USED'
                  ENDIF
                  P=P+DP*ADJ
                  CALL CLIPP(P)
                  IF (NOGO.EQ.0) GOTO 50
                  CALL SUB(P,T)
            ELSEIF(RHO.LT.DG(PS)) THEN
  60              N=N+1
                  IFLAG=2
                  RHO1=DGSUP(P,T)
                  DR=RHO-RHO1
                  DP=DR/DRGDPT(P,T)
                  ERROR=AMAX1(ABS(DR/RHO),ABS(DP/P))
                  IF(ERROR.LE.ERRTOL) NOGO=1
                  IF(N.GE.NMAX) THEN
                       NOGO=1
                  ERRMSG=' MAXIMUM NUMBER OF ITERATIONS USED'
                  ENDIF
                  P=P+DP*ADJ
                  CALL CLIPP(P)
                  IF (NOGO.EQ.0) GOTO 60
                  CALL SUP(P,T)
            ELSE
                  IFLAG=1
                  P=PS
                  CALL SAT(P,1)
            ENDIF
************************************************************************
*     h & P GIVEN
************************************************************************
      ELSEIF (ISAT.EQ.6) THEN
            N=0
            NOGO=0
            IF(SPECH.LT.HF(P)) THEN
  70              N=N+1
                  IFLAG=0
                  SPECH1=HFSUB(P,T)
                  DH=SPECH-SPECH1
                  DT=DH/DHFDTP(P,T)
                  ERROR=AMAX1(ABS(DH/SPECH),ABS(DT/T))
                  IF(ERROR.LE.ERRTOL) NOGO=1
                  IF(N.GE.NMAX) THEN
                       NOGO=1
                  ERRMSG=' MAXIMUM NUMBER OF ITERATIONS USED'
                  ENDIF
                  T=T+DT*ADJ
                  CALL CLIPT(T)
                  IF (NOGO.EQ.0) GOTO 70
                  CALL SUB(P,T)
            ELSEIF(SPECH.GT.HG(P)) THEN
  80              N=N+1
                  IFLAG=2
                  SPECH1=HGSUP(P,T)
                  DH=SPECH-SPECH1
                  DT=DH/DHGDTP(P,T)
                  ERROR=AMAX1(ABS(DH/SPECH),ABS(DT/T))
                  IF(ERROR.LE.ERRTOL) NOGO=1
                  IF(N.GE.NMAX) THEN
                       NOGO=1
                  ERRMSG=' MAXIMUM NUMBER OF ITERATIONS USED'
                  ENDIF
                  T=T+DT*ADJ
                  CALL CLIPT(T)
                  IF (NOGO.EQ.0) GOTO 80
                  CALL SUP(P,T)
            ELSE
                  T=TSAT(P)
                  IFLAG=1
                  CALL SAT(P,2)
            ENDIF
************************************************************************
*     h & T GIVEN
************************************************************************
      ELSEIF (ISAT.EQ.7) THEN
            PS=PSAT(T)
            N=0
            NOGO=0
            IF(SPECH.LT.HF(PS)) THEN
  90              N=N+1
                  IFLAG=0
                  SPECH1=HFSUB(P,T)
                  DH=SPECH-SPECH1
                  DP=DH/DHFDPT(P,T)
                  ERROR=AMAX1(ABS(DH/SPECH),ABS(DP/P))
                  IF(ERROR.LE.ERRTOL) NOGO=1
                  IF(N.GE.NMAX) THEN
                       NOGO=1
                  ERRMSG=' MAXIMUM NUMBER OF ITERATIONS USED'
                  ENDIF
                  P=P+DP*ADJ
                  CALL CLIPP(P)
                  IF (NOGO.EQ.0) GOTO 90
                  CALL SUB(P,T)
            ELSEIF(SPECH.GT.HG(PS)) THEN
  99              N=N+1
                  IFLAG=2
                  SPECH1=HGSUP(P,T)
                  DH=SPECH-SPECH1
                  DP=DH/DHGDPT(P,T)
                  ERROR=AMAX1(ABS(DH/SPECH),ABS(DP/P))
                  IF(ERROR.LE.ERRTOL) NOGO=1
                  IF(N.GE.NMAX) THEN
                       NOGO=1
                  ERRMSG=' MAXIMUM NUMBER OF ITERATIONS USED'
                  ENDIF
                  P=P+DP*ADJ
                  CALL CLIPP(P)
                  IF (NOGO.EQ.0) GOTO 99
                  CALL SUP(P,T)
            ELSE
                  IFLAG=1
                  P=PS
                  CALL SAT(P,2)
            ENDIF
      ENDIF
      NMAX=N
       RETURN
      END
************************************************************************
* SATURATED PROPERTY ROUTINE
************************************************************************
      SUBROUTINE SAT(P,ITYPE)
        COMMON/PROPERTY/VFP,VGP,RHOF,RHOG,HFP,HGP,SFP,SGP,EFP,EGP,
     *                  VOL,TOTM,TOTH,TOTS,TOTE,X,VOID,RHO,SPECVOL,
     *                  SPECH,SPECS,SPECE,PS,TS
      COMMON/SLOPES/SRFDP,SRFDT,SRGDP,SRGDT,SHFDP,SHFDT,
     *                  SHGDP,SHGDT,G1P,G2P,G1T,G2T,F1,F2,F3,F4,F5
     *                  ,STDP,SPDT,SRDP,SRDT,SHDP,SHDT
     *            ,SSFDP,SSFDT,SSGDP,SSGDT,SSDP,SSDT
      COMMON/TRANSPT/CPFP,CPGP,VISCFP,VISCGP,BETAFP,BETAGP,AKFP,AKGP,
     *            CP,VISC,BETA,AK,SPEEDFP,SPEEDGP,SPEED,CVFP,CVGP,
     *            CV
      COMMON/ERRORS/ERRH2O
      CHARACTER ERRH2O*70
************************************************************************
*     FLAGS:
*           ITYPE - 0=ONLY PRESSURE AND TEMPERATURE KNOWN
*                 1=DENSITY KNOWN
*                 2=SPECIFIC ENTHALPY KNOWN
*                 3=QUALITY KNOWN
*                 4=VOID KNOWN
************************************************************************
        VFP=VF(P)
        VGP=VG(P)
        HFP=HF(P)
        HGP=HG(P) 
        SFP=SF(P)
        SGP=SG(P) 
        EFP=HF(P)-P*VFP*1000
        EGP=HG(P)-P*VGP*1000 
      RHOG=1./VGP
      RHOF=1./VFP
************************************************************************
* IF DENSITY, ENTHALPY, QUALITY OR VOID NOT SPECIFIED, THEN CAN'T COMPLETE
* CALCULATIONS----assume single phase liquid and proceed
************************************************************************
      IF (ITYPE.EQ.0) THEN
              X=0.0
              VOID=0.0
              RHO=RHOF
              SPECVOL=VFP
              SPECH=HFP
              SPECE=EFP
              SPECS=SFP
            MSG=' PHASE QUANTITIES CALCULATED BASED ON X=0'
************************************************************************
* DENSITY KNOWN
************************************************************************
      ELSEIF (ITYPE.EQ.1) THEN
* CALCULATE OVERALL SPECIFIC QUANTITIES
              VOID=(RHOF-RHO)/(RHOF-RHOG)
              SPECVOL=1.0/RHO
            X=(VFP-SPECVOL)/(VFP-VGP)
              SPECH=HFP + X * (HGP-HFP)
              SPECE=EFP + X * (EGP-EFP)
              SPECS=SFP + X * (SGP-SFP)

************************************************************************
* SPECIFIC ENTHALPY KNOWN
************************************************************************
      ELSEIF (ITYPE.EQ.2) THEN
* CALCULATE OVERALL SPECIFIC QUANTITIES
            X=(SPECH-HFP)/(HGP-HFP)
              SPECVOL=VFP + X * (VGP-VFP)
              RHO=1/SPECVOL
              VOID=(RHOF-RHO)/(RHOF-RHOG)
              SPECE=EFP + X * (EGP-EFP)
              SPECS=SFP + X * (SGP-SFP)

************************************************************************
* QUALITY KNOWN
************************************************************************
      ELSEIF (ITYPE.EQ.3) THEN
* CALCULATE OVERALL SPECIFIC QUANTITIES
              SPECVOL=VFP + X * (VGP-VFP)
              RHO=1/SPECVOL
              VOID=(RHOF-RHO)/(RHOF-RHOG)
              SPECH=HFP + X * (HGP-HFP)
              SPECE=EFP + X * (EGP-EFP)
              SPECS=SFP + X * (SGP-SFP)

************************************************************************
* VOID KNOWN
************************************************************************
      ELSEIF (ITYPE.EQ.4) THEN
* CALCULATE OVERALL SPECIFIC QUANTITIES
            RHO=RHOF + VOID * (RHOG-RHOF)
            SPECVOL=1/RHO
            X=(VFP-SPECVOL)/(VFP-VGP)
              SPECH=HFP + X * (HGP-HFP)
              SPECE=EFP + X * (EGP-EFP)
              SPECS=SFP + X * (SGP-SFP)
      ENDIF
* CALCULATE TOTAL QUANTITIES
        TOTM=RHO*VOL
        TOTH=SPECH*TOTM
        TOTE=SPECE*TOTM
        TOTS=SPECS*TOTM

* CALCULATE SLOPES ALONG SATURATION LINE
      TS    = TSAT(P)
      STDP  = DTDP(P)
      SPDT  = DPDT(TS)
        SHFDP = DHFDP(P)
      SVFDP = DVFDP(P)
      SRFDP = -RHOF**2 * DVFDP(P)
      SSFDP = DSFDP(P)
        SHGDP = DHGDP(P)
      SVGDP = DVGDP(P)
      SRGDP = -RHOG**2 * DVGDP(P)
      SSGDP = DSGDP(P)

        SHFDT = DHFDP(P) * SPDT
      SRFDT = SRFDP    * SPDT
        SHGDT = DHGDP(P) * SPDT
      SRGDT = SRGDP    * SPDT
      SSFDT = DSFDP(P) * SPDT
      SSGDT = DSGDP(P) * SPDT
        SHDP  = SHFDP * (1-X) + SHGDP * X - (HGP-HFP)/(VGP-VFP) *
     *      ( X * (SVGDP - SVFDP) + SVFDP)
      SSDP  = SSFDP * (1-X) + SSGDP * X - (SGP-SFP)/(VGP-VFP) *
     *      ( X * (SVGDP - SVFDP) + SVFDP)
        SVDP  = SVFDP * (1-X) + SVGDP * X - (VGP-VFP)/(HGP-HFP) *
     *      ( X * (SHGDP - SHFDP) + SHFDP)
      SRDP  = -RHO*RHO*SVDP
      SRDT  = SRDP * SPDT
      SHDT  = SHDP * SPDT
      SSDT  = SSDP * SPDT
* CALCULATE F FUNCTIONS
        F1= HGP*VFP -HFP*VGP
        F2= VGP-VFP 
        F3= HFP -HGP
        F4= SHGDP*( VGP-VFP) - SVGDP*( HGP-HFP)
        F5= SHFDP*( VGP-VFP) - SVFDP*( HGP-HFP)

* CALCULATE G FUNCTIONS
      DENOM= X * F4 + (1-X) * F5
      G1P= -F3 * SPECVOL**2 / DENOM
      G2P= F2 / DENOM
      G1T= G1P * STDP
      G2T= G2P * STDP

* CALCULATE BETA
      BETA = -SRDT/RHO
      BETAFP = -SRFDT/RHOF
      BETAGP = -SRGDT/RHOG

* CALCULATE AK
      AK = SRDP/RHO * 1.E-06
      AKFP = SRFDP/RHOF * 1.E-06
      AKGP = SRGDP/RHOG * 1.E-06

* CALCULATE Cp
      CPFP = CPF(P)
      CPGP = CPG(P)
*                      MASS WEIGHTED HEAT CAPACITY
      CP = CPFP*(1-X) + CPGP*X

* CALCULATE Cv 
      TK = T+273.16
      CVFP = CPFP - BETAFP**2 * TK * VFP / AKFP *1.E-03
      CVGP = CPGP - BETAGP**2 * TK * VGP / AKGP *1.E-03
*                      MASS WEIGHTED HEAT CAPACITY
      CV = CVFP*(1-X) + CVGP*X

* CALCULATE DYNAMIC VISCOSITY
      VISCFP = VISCF(P)
      VISCGP = VISCG(P)
*                      MASS WEIGHTED VISCOSITY ????????
      VISC   = (1-X) * VISCFP + X * VISCGP

* CALCULATE SPEED OF SOUND 
      RATIO = CPFP/TK
      SPEEDFP=SQRT( ABS(RATIO/(SRFDP*1.E-06*RATIO - 
     *      (VFP*SRFDT)**2*1.E-03)))
      RATIO = CPGP/TK
      SPEEDGP=SQRT( ABS(RATIO/(SRGDP*1.E-06*RATIO - 
     *      (VGP*SRGDT)**2*1.E-03)))
      RATIO = CP/TK
      SPEED=SQRT( ABS(RATIO/(SRDP*1.E-06*RATIO - 
     *      (SPECVOL*SRDT)**2*1.E-03 )))
        RETURN
        END
************************************************************************
* SUBCOOLED PROPERTY ROUTINE
************************************************************************
      SUBROUTINE SUB(P,T)
        COMMON/PROPERTY/VFP,VGP,RHOF,RHOG,HFP,HGP,SFP,SGP,EFP,EGP,
     *                  VOL,TOTM,TOTH,TOTS,TOTE,X,VOID,RHO,SPECVOL,
     *                  SPECH,SPECS,SPECE,PS,TS
      COMMON/SLOPES/SRFDP,SRFDT,SRGDP,SRGDT,SHFDP,SHFDT,
     *                  SHGDP,SHGDT,G1P,G2P,G1T,G2T,F1,F2,F3,F4,F5
     *                  ,STDP,SPDT,SRDP,SRDT,SHDP,SHDT
     *            ,SSFDP,SSFDT,SSGDP,SSGDT,SSDP,SSDT
      COMMON/TRANSPT/CPFP,CPGP,VISCFP,VISCGP,BETAFP,BETAGP,AKFP,AKGP,
     *            CP,VISC,BETA,AK,SPEEDFP,SPEEDGP,SPEED,CVFP,CVGP,
     *            CV
      COMMON/ERRORS/ERRH2O
      CHARACTER ERRH2O*70

* SUBCOOLED, THEREFORE NO VOID OR QUALITY
        X=0.0
        VOID=0.0

* CALCULATE PHASE PROPERTIES
        VFP=VFSUB(P,T)
        VGP=0.0
        HFP=HFSUB(P,T)
        HGP=0.0 
        SFP=SFSUB(P,T)
        SGP=0.0 
        EFP=HFP-P*VFP*1000
        EGP=0.0 
      RHOF=1./VFP
      RHOG=0.0

* CALCULATE OVERALL SPECIFIC QUANTITIES
        RHO=RHOF
        SPECVOL=VFP
        SPECH=HFP
        SPECE=EFP
        SPECS=SFP

* CALCULATE TOTAL QUANTITIES
        TOTM=RHO*VOL
        TOTH=SPECH*TOTM
        TOTE=SPECE*TOTM
        TOTS=SPECS*TOTM

* CALCULATE SLOPES 
      SRFDT = DRFDTP(P,T)
      SHFDT = DHFDTP(P,T)
      SSFDT = DSFDTP(P,T)
      SRFDP = DRFDPT(P,T)
      SHFDP = DHFDPT(P,T)
      SSFDP = DSFDPT(P,T)
      SRGDT = 0.0
      SHGDT = 0.0
      SRGDP = 0.0
      SHGDP = 0.0
      SSGDT = 0.0
      SSGDP = 0.0
      SPDT  = DPDT(T)
      STDP  = DTDP(P)
      SHDP  = SHFDP
      SHDT  = SHFDT
      SRDP  = SRFDP
      SRDT  = SRFDT
      SSDP  = SSFDP
      SSDT  = SSFDT
* CALCULATE F FUNCTIONS
        F1= RHO * SHFDT + SPECH * SRFDT
        F2= - SRFDT
        F3= - RHO **2 * SHFDT
        F4= 0.0
        F5= SRFDP * SHFDT - SRFDT * SHFDP
* CALCULATE G FUNCTIONS
      G1P= SHFDT / F5 
      G2P= -SRFDT / F5
      G1T= -SHFDP / F5 
      G2T= SRFDP / F5
* CALCULATE BETA
      BETAFP = -SRFDT/RHOF
      BETAGP = 0.0
      BETA = BETAFP
* CALCULATE AK
      AKFP = SRFDP/RHOF * 1.E-06
      AKGP = 0.0
      AK = AKFP
* CALCULATE CP
      CPFP = CPFSUB(P,T)
      CPGP = 0.0
      CP = CPFP
* CALCULATE CV 
      TK = T+273.16
      CVFP = CPFP - BETAFP**2 * TK * VFP / AKFP *1.E-03
      CVGP = 0.0
      CV = CVFP
* CALCULATE DYNAMIC VISCOSITY (ASSUME SATURATED)
      VISCFP = VISCF(P)
      VISCGP = 0.0
      VISC   = VISCFP
* CALCULATE SPEED OF SOUND 
      RATIO = CPFP/TK
        SPEEDFP=SQRT( ABS(RATIO/(SRFDP*1.E-06*RATIO - 
     *      (VFP*SRFDT)**2*1.E-03)))
      SPEEDGP = 0.0
      SPEED = SPEEDFP
        RETURN
        END
************************************************************************
* SUPERHEATED PROPERTY ROUTINE
************************************************************************
      SUBROUTINE SUP(P,T)
        COMMON/PROPERTY/VFP,VGP,RHOF,RHOG,HFP,HGP,SFP,SGP,EFP,EGP,
     *                  VOL,TOTM,TOTH,TOTS,TOTE,X,VOID,RHO,SPECVOL,
     *                  SPECH,SPECS,SPECE,PS,TS
      COMMON/SLOPES/SRFDP,SRFDT,SRGDP,SRGDT,SHFDP,SHFDT,
     *                  SHGDP,SHGDT,G1P,G2P,G1T,G2T,F1,F2,F3,F4,F5
     *                  ,STDP,SPDT,SRDP,SRDT,SHDP,SHDT
     *            ,SSFDP,SSFDT,SSGDP,SSGDT,SSDP,SSDT
      COMMON/TRANSPT/CPFP,CPGP,VISCFP,VISCGP,BETAFP,BETAGP,AKFP,AKGP,
     *            CP,VISC,BETA,AK,SPEEDFP,SPEEDGP,SPEED,CVFP,CVGP,
     *            CV
      COMMON/ERRORS/ERRH2O
      CHARACTER ERRH2O*70
* SUPERHEATED, THEREFORE ALL VAPOUR
        X=1.0
        VOID=1.0

* CALCULATE PHASE PROPERTIES
        VFP=0.0
        VGP=VGSUP(P,T)
        HFP=0.0
        HGP=HGSUP(P,T) 
        SFP=0.0
        SGP=SGSUP(P,T) 
        EFP=0.0
        EGP=HGP-P*VGP*1000 
      RHOF=0.0
      RHOG=1./VGP

* CALCULATE OVERALL SPECIFIC QUANTITIES
        RHO=RHOG
        SPECVOL=VGP
        SPECH=HGP
        SPECE=EGP
        SPECS=SGP

* CALCULATE TOTAL QUANTITIES
        TOTM=RHO*VOL
        TOTH=SPECH*TOTM
        TOTE=SPECE*TOTM
        TOTS=SPECS*TOTM

* CALCULATE SLOPES
      SRGDT = DRGDTP(P,T)
      SHGDT = DHGDTP(P,T)
      SSGDT = DSGDTP(P,T)
      SRGDP = DRGDPT(P,T)
      SHGDP = DHGDPT(P,T)
      SSGDP = DSGDPT(P,T)
      SRFDT = 0.0
      SHFDT = 0.0
      SSFDT = 0.0
      SRFDP = 0.0
      SHFDP = 0.0
      SSFDP = 0.0
      SPDT  = DPDT(T)
      STDP  = DTDP(P)
      SHDP  = SHGDP
      SHDT  = SHGDT
      SRDP  = SRGDP
      SRDT  = SRGDT
      SSDP  = SSGDP
      SSDT  = SSGDT
* CALCULATE F FUNCTIONS
        F1= RHO * SHGDT + SPECH * SRGDT
        F2= - SRGDT
        F3= - RHO **2 * SHGDT
        F4= SRGDP * SHGDT - SRGDT * SHGDP
        F5= 0.0
* CALCULATE G FUNCTIONS
      G1P= SHGDT / F4 
      G2P= -SRGDT / F4
      G1T= -SHGDP / F4
      G2T= SRGDP / F4
* CALCULATE BETA
      BETAFP = 0.0
      BETAGP = -SRGDT/RHOG
      BETA = BETAGP
* CALCULATE AK
      AKFP = 0.0
      AKGP = SRGDP/RHOG *1.E-06
      AK = AKGP
* CALCULATE CP
      CPFP = 0.0
      CPGP = CPGSUP(P,T)
      CP = CPGP
* CALCULATE CV 
      TK = T+273.16
      CVFP = 0.0
      CVGP = CPGP - BETAGP**2 * TK * VGP / AKGP *1.E-03
      CV = CVGP
* CALCULATE DYNAMIC VISCOSITY (ASSUME SATURATED)
      VISCFP = 0.0
      VISCGP = VISCG(P)
      VISC   = VISCGP
* CALCULATE SPEED OF SOUND 
      RATIO = CPGP/TK
      SPEEDFP = 0.0
        SPEEDGP=SQRT( ABS(RATIO/(SRGDP*1.E-06*RATIO - 
     *      (VGP*SRGDT)**2*1.E-03)))
      SPEED = SPEEDGP
        RETURN
        END

************************************************************************
*  dh/dP |sat FUNCTION
************************************************************************
      SUBROUTINE DHDP(P,T,X,SHDP)
      COMMON/ERRORS/ERRH2O
      CHARACTER ERRH2O*70
* CALCULATE PROPERTIES
        VFP=VF(P)
        VGP=VG(P)
        HFP=HF(P)
        HGP=HG(P) 
* CALCULATE SLOPES ALONG SATURATION LINE
        SHFDP = DHFDP(P)
      SVFDP = DVFDP(P)
        SHGDP = DHGDP(P)
      SVGDP = DVGDP(P)
        SHDP  = SHFDP * (1-X) + SHGDP * X - (HGP-HFP)/(VGP-VFP) *
     *      ( X * (SVGDP - SVFDP) + SVFDP)
        RETURN
        END

************************************************************************
* SATURATED F ROUTINE
************************************************************************
      SUBROUTINE FPSAT(P,T,F1,F2,F3,F4,F5)
      COMMON/ERRORS/ERRH2O
      CHARACTER ERRH2O*70
* CALCULATE PROPERTIES
        VFP=VF(P)
        VGP=VG(P)
        HFP=HF(P)
        HGP=HG(P) 
* CALCULATE SLOPES ALONG SATURATION LINE
        SHFDP = DHFDP(P)
      SVFDP = DVFDP(P)
        SHGDP = DHGDP(P)
      SVGDP = DVGDP(P)
* CALCULATE F FUNCTIONS
        F1= HGP*VFP -HFP*VGP
        F2= VGP-VFP 
        F3= HFP -HGP
        F4= SHGDP*( VGP-VFP) - SVGDP*( HGP-HFP)
        F5= SHFDP*( VGP-VFP) - SVFDP*( HGP-HFP)
        RETURN
        END

************************************************************************
* SUBCOOLED F ROUTINE
************************************************************************
      SUBROUTINE FPSUB(P,T,F1,F2,F3,F4,F5)
      COMMON/ERRORS/ERRH2O
      CHARACTER ERRH2O*70
* CALCULATE SLOPES 
      SRFDT = DRFDTP(P,T)
      SHFDT = DHFDTP(P,T)
      SRFDP = DRFDPT(P,T)
      SHFDP = DHFDPT(P,T)
* CALCULATE F FUNCTIONS
        F1= RHO * SHFDT + SPECH * SRFDT
        F2= - SRFDT
        F3= - RHO **2 * SHFDT
        F4= 0.0
        F5= SRFDP * SHFDT - SRFDT * SHFDP
      RETURN
      END

************************************************************************
* SUPERHEATED F ROUTINE
************************************************************************
      SUBROUTINE FPSUP(P,T,F1,F2,F3,F4,F5)
      COMMON/ERRORS/ERRH2O
      CHARACTER ERRH2O*70
* CALCULATE SLOPES 
      SRGDT = DRGDTP(P,T)
      SHGDT = DHGDTP(P,T)
      SRGDP = DRGDPT(P,T)
      SHGDP = DHGDPT(P,T)
* CALCULATE F FUNCTIONS
        F1= RHO * SHGDT + SPECH * SRGDT
        F2= - SRGDT 
        F3= - RHO **2 * SHGDT
        F4= SRGDP * SHGDT - SRGDT * SHGDP
        F5= 0.0
      RETURN
      END

************************************************************************
* SATURATED G ROUTINES
************************************************************************
      SUBROUTINE GPSAT(P,T,X,G1,G2)
      COMMON/ERRORS/ERRH2O
      CHARACTER ERRH2O*70
* CALCULATE PROPERTIES
        VFP=VF(P)
        VGP=VG(P)
        HFP=HF(P)
        HGP=HG(P) 
* CALCULATE SLOPES ALONG SATURATION LINE
        SHFDP = DHFDP(P)
      SVFDP = DVFDP(P)
        SHGDP = DHGDP(P)
      SVGDP = DVGDP(P)
* CALCULATE F FUNCTIONS
        F2= VGP-VFP 
        F3= HFP -HGP
        F4= SHGDP*( VGP-VFP) - SVGDP*( HGP-HFP)
        F5= SHFDP*( VGP-VFP) - SVFDP*( HGP-HFP)
* CALCULATE G FUNCTIONS
      DENOM= X * F4 + (1-X) * F5
      G1= -F3 * SPECVOL**2 / DENOM
      G2= F2 / DENOM
        RETURN
        END

      SUBROUTINE GTSAT(P,T,X,G1,G2)
      COMMON/ERRORS/ERRH2O
      CHARACTER ERRH2O*70
      CALL GPSAT(P,T,X,G1,G2)
      STDP = DTDP(P)
      G1=G1 * STDP
      G2=G2 * STDP
        RETURN
        END

************************************************************************
* SUBCOOLED G ROUTINES
************************************************************************
*----------P & T calculations combined-------------
      SUBROUTINE GSUB(P,T,GP1,GP2,GT1,GT2)
      COMMON/ERRORS/ERRH2O
      CHARACTER ERRH2O*70
* CALCULATE SLOPES 
      SRFDT = DRFDTP(P,T)
      SHFDT = DHFDTP(P,T)
      SRFDP = DRFDPT(P,T)
      SHFDP = DHFDPT(P,T)
        F5= SRFDP * SHFDT - SRFDT * SHFDP
* CALCULATE GP FUNCTIONS
      GP1= SHFDT / F5 
      GP2= -SRFDT / F5
        F5= -SRFDP * SHFDT + SRFDT * SHFDP
* CALCULATE GT FUNCTIONS
      GT1= SHFDP / F5 
      GT2= -SRFDP / F5
      RETURN
      END

*----------P  calculation-------------
      SUBROUTINE GPSUB(P,T,G1,G2)
      COMMON/ERRORS/ERRH2O
      CHARACTER ERRH2O*70
* CALCULATE SLOPES 
      SRFDT = DRFDTP(P,T)
      SHFDT = DHFDTP(P,T)
      SRFDP = DRFDPT(P,T)
      SHFDP = DHFDPT(P,T)
        F5= SRFDP * SHFDT - SRFDT * SHFDP
* CALCULATE G FUNCTIONS
      G1= SHFDT / F5 
      G2= -SRFDT / F5
      RETURN
      END

*----------T  calculation-------------
      SUBROUTINE GTSUB(P,T,G1,G2)
      COMMON/ERRORS/ERRH2O
      CHARACTER ERRH2O*70
* CALCULATE SLOPES 
      SRFDT = DRFDTP(P,T)
      SHFDT = DHFDTP(P,T)
      SRFDP = DRFDPT(P,T)
      SHFDP = DHFDPT(P,T)
        F5= -SRFDP * SHFDT + SRFDT * SHFDP
* CALCULATE G FUNCTIONS
      G1= SHFDP / F5 
      G2= -SRFDP / F5
      RETURN
      END

************************************************************************
* SUPERHEATED G ROUTINES
************************************************************************
*----------P & T calculations combined-------------
      SUBROUTINE GSUP(P,T,GP1,GP2,GT1,GT2)
      COMMON/ERRORS/ERRH2O
      CHARACTER ERRH2O*70
* CALCULATE SLOPES 
      SRGDT = DRGDTP(P,T)
      SHGDT = DHGDTP(P,T)
      SRGDP = DRGDPT(P,T)
      SHGDP = DHGDPT(P,T)
        F4= SRGDP * SHGDT - SRGDT * SHGDP
* CALCULATE GP FUNCTIONS
      GP1= SHGDT / F4 
      GP2= -SRGDT / F4
        F4= -SRGDP * SHGDT + SRGDT * SHGDP
* CALCULATE GT FUNCTIONS
      GT1= SHGDP / F4 
      GT2= -SRGDP / F4
      RETURN
      END

*----------P  calculation-------------
      SUBROUTINE GPSUP(P,T,G1,G2)
      COMMON/ERRORS/ERRH2O
      CHARACTER ERRH2O*70
* CALCULATE SLOPES 
      SRGDT = DRGDTP(P,T)
      SHGDT = DHGDTP(P,T)
      SRGDP = DRGDPT(P,T)
      SHGDP = DHGDPT(P,T)
        F4= SRGDP * SHGDT - SRGDT * SHGDP
* CALCULATE G FUNCTIONS
      G1= SHGDT / F4 
      G2= -SRGDT / F4
      RETURN
      END

*----------T  calculation-------------
      SUBROUTINE GTSUP(P,T,G1,G2)
      COMMON/ERRORS/ERRH2O
      CHARACTER ERRH2O*70
* CALCULATE SLOPES 
      SRGDT = DRGDTP(P,T)
      SHGDT = DHGDTP(P,T)
      SRGDP = DRGDPT(P,T)
      SHGDP = DHGDPT(P,T)
        F4= -SRGDP * SHGDT + SRGDT * SHGDP
* CALCULATE G FUNCTIONS
      G1= SHGDP / F4 
      G2= -SRGDP / F4
      RETURN
      END

************************************************************************
*  Clip routines for pressure, temperature, enthalpy and density
************************************************************************
      SUBROUTINE CLIPP(P)
C PMIN changed from   .065.  rjw 4 April 1991
C PMAX chamged from  21.55.  rjw 4 April 1991
      PMIN=0.002
      PMAX=21.50
      P=AMIN1(P,PMAX)
      P=AMAX1(P,PMIN)
      RETURN
      END

      SUBROUTINE CLIPT(T)
C TMIN changed from 89.  rjw 4 April 1991
      TMIN=18.
      TMAX=450.
      T=AMIN1(T,TMAX)
      T=AMAX1(T,TMIN)
      RETURN
      END

      SUBROUTINE CLIPH(H)
      HMIN=.0
      HMAX=3000.
      H=AMIN1(H,HMAX)
      H=AMAX1(H,HMIN)
      RETURN
      END

      SUBROUTINE CLIPR(R)
      RMIN=.0001
      RMAX=1100.
      R=AMIN1(R,RMAX)
      R=AMAX1(R,RMIN)
      RETURN
      END
